import { CompareFn } from '..';
import { LinkedNode } from "./linkedNode";
/**
 * Groups an iterable into batches of a given size.
 *
 * @param size - The maximum batch size
 * @param iterable - The iterable to batch
 *
 * @returns An iterable of batches
 *
 * @internal
 */
export declare function batch<T>(size: number, iterable: Iterable<T>): Generator<T[], void, number | undefined>;
/**
 * Groups an iterable into batches of a given size.
 *
 * @param size - The maximum batch size
 * @param iterable - The iterable to batch
 *
 * @returns An iterable of batches
 *
 * @internal
 */
export declare function batchArray<T>(size: number, array: T[]): Generator<T[], void, number | undefined>;
/**
 * Groups an iterable into batches of a given size.
 *
 * @param size - The maximum batch size
 * @param iterable - The iterable to batch
 *
 * @returns An iterable of batches
 *
 * @internal
 */
export declare function batchIterable<T>(size: number, iterable: Iterable<T>): Generator<T[], void, number | undefined>;
/**
 * Limits a number to be within a given range.
 *
 * f(x, min, max) = y, where min <= y <= max
 *
 * @param num - The number to clamp
 * @param min - The minimum result value, inclusive
 * @param max - The maximum result value, inclusive
 *
 * @returns The clamped number
 *
 * @internal
 */
export declare function clamp(num: number, min: number, max: number): number;
/**
 * Wraps and then clamps a number within a given range.
 *
 * @param num - The number to wrap and then clamp
 * @param min - The minimum result value, inclusive
 * @param max - The wrap pivot and maximum result value, inclusive
 *
 * @returns The crwapped number
 *
 * @internal
 */
export declare function cwrap(num: number, min: number, max: number): number;
/**
 * Sorts a list in place.
 *
 * Works on complete lists as well as sublists and circular lists:
 * - Linked lists will keep the link to the next node beyond the sorted section
 * - Doubly linked lists will keep links to the prev and next nodes outside the sorted section
 *
 * @param node - The head of the list
 * @param len - The length of the list beginning from node
 * @param isDoubly - Whether node is a doubly linked node
 * @param compareFn - A function used to determine the order of elements.
 *
 * It is expected to return:
 * - A negative value if first argument < second argument
 * - Zero if first argument == second argument
 * - A positive value if first argument > second argument
 *
 * @returns The new head and tail of the sorted list
 *
 * @internal
 */
export declare function mergeSort<T, Node extends LinkedNode<T>>(node: Node, len: number, isDoubly: boolean, compareFn: CompareFn<T>): [Node, Node];
/**
 * Merges two sorted lists.
 *
 * @param nodes - The heads of the lists
 * @param lens - The lengths of the lists
 * @param isDoubly - Whether the lists are a doubly linked
 * @param compareFn - A function used to determine the order of elements.
 *
 * It is expected to return:
 * - A negative value if first argument < second argument
 * - Zero if first argument == second argument
 * - A positive value if first argument > second argument
 *
 * @returns The new head of the sorted list
 *
 * @internal
 */
export declare function mergeSorted<T, Node extends LinkedNode<T>>(heads: [Node, Node], lens: [number, number], isDoubly: boolean, compareFn: CompareFn<T>): Node;
/**
 * Wraps a number around a pivot
 *
 * f(x, min, pivot) = {
 *    x, where x >= 0
 *    pivot + x, where x < 0
 * }
 *
 * @param num - The number to wrap
 * @param pivot - The number to pivot on
 *
 * @returns - The wrapped number
 *
 * @internal
 */
export declare function wrap(num: number, pivot: number): number;
/**
 * Moves a given number of spaces forward in a list
 *
 * @param head - The head of the list
 * @param len - The number of spaces to move
 *
 * @returns - The node len spaces ahead from head
 *
 * @internal
 *
export function next<T, Node extends LinkedNode<T>>(head: Node, len: number): Node {
    while (len-- > 0) {
        head = head.next as Node;
    }
    return head;
}
*/ 
