"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrap = exports.mergeSorted = exports.mergeSort = exports.cwrap = exports.clamp = exports.batchIterable = exports.batchArray = exports.batch = void 0;
const utils_1 = require("../array/utils");
/**
 * Groups an iterable into batches of a given size.
 *
 * @param size - The maximum batch size
 * @param iterable - The iterable to batch
 *
 * @returns An iterable of batches
 *
 * @internal
 */
function batch(size, iterable) {
    if (utils_1.isArray(iterable)) {
        return batchArray(size, iterable);
    }
    return batchIterable(size, iterable);
}
exports.batch = batch;
/**
 * Groups an iterable into batches of a given size.
 *
 * @param size - The maximum batch size
 * @param iterable - The iterable to batch
 *
 * @returns An iterable of batches
 *
 * @internal
 */
function* batchArray(size, array) {
    let min = 0;
    while (min < array.length) {
        const max = min + size;
        size = (yield array.slice(min, max)) ?? size;
        min = max;
    }
}
exports.batchArray = batchArray;
/**
 * Groups an iterable into batches of a given size.
 *
 * @param size - The maximum batch size
 * @param iterable - The iterable to batch
 *
 * @returns An iterable of batches
 *
 * @internal
 */
function* batchIterable(size, iterable) {
    let array = [];
    for (const value of iterable) {
        if (array.push(value) >= size) {
            size = (yield array) ?? size;
            array = [];
        }
    }
    if (array.length > 0) {
        yield array;
    }
}
exports.batchIterable = batchIterable;
/**
 * Limits a number to be within a given range.
 *
 * f(x, min, max) = y, where min <= y <= max
 *
 * @param num - The number to clamp
 * @param min - The minimum result value, inclusive
 * @param max - The maximum result value, inclusive
 *
 * @returns The clamped number
 *
 * @internal
 */
function clamp(num, min, max) {
    return Math.min(max, Math.max(min, num));
}
exports.clamp = clamp;
/**
 * Wraps and then clamps a number within a given range.
 *
 * @param num - The number to wrap and then clamp
 * @param min - The minimum result value, inclusive
 * @param max - The wrap pivot and maximum result value, inclusive
 *
 * @returns The crwapped number
 *
 * @internal
 */
function cwrap(num, min, max) {
    return clamp(wrap(num, max), min, max);
}
exports.cwrap = cwrap;
/**
 * Sorts a list in place.
 *
 * Works on complete lists as well as sublists and circular lists:
 * - Linked lists will keep the link to the next node beyond the sorted section
 * - Doubly linked lists will keep links to the prev and next nodes outside the sorted section
 *
 * @param node - The head of the list
 * @param len - The length of the list beginning from node
 * @param isDoubly - Whether node is a doubly linked node
 * @param compareFn - A function used to determine the order of elements.
 *
 * It is expected to return:
 * - A negative value if first argument < second argument
 * - Zero if first argument == second argument
 * - A positive value if first argument > second argument
 *
 * @returns The new head and tail of the sorted list
 *
 * @internal
 */
function mergeSort(node, len, isDoubly, compareFn) {
    // Base case
    if (len < 2) {
        return [node, node];
    }
    // Split the list into two halves and sort them
    len = len / 2;
    const lens = [Math.ceil(len), Math.floor(len)];
    const heads = mergeSort(node, lens[0], isDoubly, compareFn);
    const tails = mergeSort(heads[1].next, lens[1], isDoubly, compareFn);
    // Group the heads and tails together
    node = heads[1];
    heads[1] = tails[0];
    tails[0] = node;
    tails[0].next = tails[1].next;
    // Merge the sorted halves
    const prev = heads[0].prev;
    node = mergeSorted(heads, lens, isDoubly, compareFn);
    isDoubly && (node.prev = prev);
    // Return the head and tail
    return [node, tails[+(lens[0] < 1)]];
}
exports.mergeSort = mergeSort;
/**
 * Merges two sorted lists.
 *
 * @param nodes - The heads of the lists
 * @param lens - The lengths of the lists
 * @param isDoubly - Whether the lists are a doubly linked
 * @param compareFn - A function used to determine the order of elements.
 *
 * It is expected to return:
 * - A negative value if first argument < second argument
 * - Zero if first argument == second argument
 * - A positive value if first argument > second argument
 *
 * @returns The new head of the sorted list
 *
 * @internal
 */
function mergeSorted(heads, lens, isDoubly, compareFn) {
    const root = {};
    let node = root;
    do {
        const index = +(compareFn(heads[0].value, heads[1].value) > 0);
        node.next = heads[index];
        isDoubly && (node.next.prev = node);
        node = node.next;
        heads[index] = node.next;
        --lens[index];
    } while (lens[0] > 0 && lens[1] > 0);
    // Add any remaining nodes
    node.next = heads[+(lens[0] < 1)];
    isDoubly && node.next && (node.next.prev = node);
    return root.next;
}
exports.mergeSorted = mergeSorted;
/**
 * Wraps a number around a pivot
 *
 * f(x, min, pivot) = {
 *    x, where x >= 0
 *    pivot + x, where x < 0
 * }
 *
 * @param num - The number to wrap
 * @param pivot - The number to pivot on
 *
 * @returns - The wrapped number
 *
 * @internal
 */
function wrap(num, pivot) {
    return num < 0 ? pivot + num : num;
}
exports.wrap = wrap;
/**
 * Moves a given number of spaces forward in a list
 *
 * @param head - The head of the list
 * @param len - The number of spaces to move
 *
 * @returns - The node len spaces ahead from head
 *
 * @internal
 *
export function next<T, Node extends LinkedNode<T>>(head: Node, len: number): Node {
    while (len-- > 0) {
        head = head.next as Node;
    }
    return head;
}
*/ 
//# sourceMappingURL=utils.js.map